################################################################################
####                              plot_avg_ir.py                            ####
################################################################################

import os
import numpy as np
import pandas as pd
import plotnine as p9

""" Plot the IR spectra (obtained with R) averaged over the time snapshots,
    comparing isotopic compositions.
"""

# CHANGE DIRECTORY!
#os.chdir("./")
os.chdir("/Users/celerity/Library/CloudStorage/Box-Box/Home Folder jzw5/" +
         "Research/quantum-olfaction/paper/data/data-plot")
dat = pd.read_csv('avg_ir.csv', sep = ',', index_col = None)
dat_gp = pd.read_csv('avg_ir_gasphase.csv', sep = ',', index_col = None)

# Add plotting columns to dat_gp
y_max = [1000 for i in range(dat_gp.shape[0])]
dat_gp['ymax'] = y_max
gp_scale = 0.5
dat_gp['freqplot'] = dat_gp['ymax'] - gp_scale * dat_gp['intensity']

def num_to_isotope(n, num_h):
  """ Given integer 0 <= n <= 2**num_h, 
      return corresponding isotopes and masses.
      Modified from the one in ash_input.py, which also returns
      the masses of the ligand in this work.
  """
  if not type(n) is int or n < 0 or not type(num_h) is int or  n > 2**num_h - 1:
    return TypeError("Input integers n, num_h such that  0 <= n <= 2**num_h.")
  
  # Binary representation: 5 digits, no leading '0b'; 0 -> h, 1 -> d
  isotopes = str(bin(n)[2:].zfill(num_h)).replace('0', 'h').replace('1', 'd')

  return isotopes

def ir_gauss(x, freq_mean, freq_sd, int_mean):
  return sum([im * np.exp(-(((x-fm)/fsd)**2))
             for fm, fsd, im in zip(freq_mean, freq_sd, int_mean)])

num_h = 5
freqs = [x for x in range(3200, 50, -1)]

# Colors: from ColorBrewer 2.0, a colorblind-friendly, 6-class diverging palate.
# Key: number of deuterium atoms (0 is bluest for hhhhh, 5 is reddest for ddddd)
iso_color = {0 : '#4575b4', 1 : '#91bfdb', 2 : '#e0f3f8', 
             3 : '#fee090', 4 : '#fc8d59', 5 : '#d73027'}

# Individual isotope plots
for n in range(2**num_h):
  iso = num_to_isotope(n, num_h)

  dat_plot = dat.where(dat.isotope == iso).dropna()
  gp_plot = dat_gp.where(dat.isotope == iso).dropna() 
  freq_mean = dat_plot.freqmean.tolist()
  freq_sd = dat_plot.freqsd.tolist()
  int_mean = dat_plot.intmean.tolist()

  ints_gauss = [ir_gauss(x, freq_mean, freq_sd, int_mean) for x in freqs]

  dat_freq = pd.DataFrame({"freq" : freqs, "int": ints_gauss})

  plot = (
  p9.ggplot(data = dat_freq, 
            mapping = p9.aes(x = "freq", y = "int")) +
  p9.geom_area(alpha = 0.4, position = "identity",
               color = "black",       # line
               fill = iso_color[iso.count('d')]) +
  p9.geom_segment(data = gp_plot, 
                  mapping = p9.aes(x = "freq", xend = "freq",
                                   y = "ymax", yend = "freqplot"),
                  color = iso_color[iso.count('d')],
                  alpha = 0.6,
                  size = 1) +
  p9.theme_classic() +
  p9.theme(plot_background = p9.element_rect(fill = "white", alpha = 0),
           panel_background = p9.element_rect(fill = "white", alpha = 0),
           panel_border = p9.element_rect(fill = "white", alpha = 0)) +
  p9.scale_x_reverse() +
  p9.scale_y_continuous(expand = (0, 0, 0, 50)) +
  p9.xlab("Wavenumber (cm$^{-1}$)") +
  p9.ylab("Intensity (arbitrary units)") +
  p9.annotate('text', x = 3075, y = 60, label = "C–H") +
  p9.annotate('text', x = 2250, y = 60, label = "C–D") +
  p9.annotate('text', x = 1600, y = 650, label = "C=O") +
  p9.annotate('text', x = 1400, y = 275, label = "C=O")
  )

  #p9.ggsave(plot, filename = "vib_" + str(iso) + "_withgasphase.pdf", 
  #          width = 12, height = 6, units = "in", dpi = 300)


# Overlaid plot: all-H vs. all-D. The others interpolate between these two
dat_h = dat.where(dat.isotope == 'hhhhh').dropna()
freq_mean_h = dat_h.freqmean.tolist()
freq_sd_h = dat_h.freqsd.tolist()
int_mean_h = dat_h.intmean.tolist()

dat_d = dat.where(dat.isotope == 'ddddd').dropna()
freq_mean_d = dat_d.freqmean.tolist()
freq_sd_d = dat_d.freqsd.tolist()
int_mean_d = dat_d.intmean.tolist()

gp_h = dat_gp.where(dat_gp.isotope == 'hhhhh').dropna()
gp_d = dat_gp.where(dat_gp.isotope == 'ddddd').dropna()

gp_h.rename(columns = {"isotope" : "Isotope"}, inplace = True)
gp_d.rename(columns = {"isotope" : "Isotope"}, inplace = True)

gp_h['Isotope'] = gp_h['Isotope'].str.replace("hhhhh", "HHHHH")
gp_d['Isotope'] = gp_d['Isotope'].str.replace("ddddd", "DDDDD")

gp_freq_allhd = pd.concat((gp_h, gp_d))

ints_gauss_h = [ir_gauss(x, freq_mean_h, freq_sd_h, int_mean_h)
                for x in freqs]
ints_gauss_d = [ir_gauss(x, freq_mean_d, freq_sd_d, int_mean_d)
                for x in freqs]

dat_freq_allhd = (pd.DataFrame({"freq" : freqs, 
                                "int_h": ints_gauss_h,
                                "int_d": ints_gauss_d})
                  .melt(id_vars = ["freq"], value_vars = ["int_h", "int_d"], 
                        var_name = "Isotope", value_name = "int")
                 )

dat_freq_allhd["Isotope"] = (dat_freq_allhd["Isotope"]
                             .str.replace("int_h", 'HHHHH')
                             .replace("int_d", 'DDDDD'))

plot = (
  p9.ggplot(data = dat_freq_allhd, 
            mapping = p9.aes(x = "freq", y = "int", group = "Isotope",
                             fill = "Isotope")) +
  p9.geom_area(alpha = 0.4, position = "identity",
               color = "black") +
  p9.geom_segment(data = gp_freq_allhd, 
                  mapping = p9.aes(x = "freq", xend = "freq",
                                   y = "ymax", yend = "freqplot",
                                   group = "Isotope",
                                   color = "Isotope"),
                  alpha = 0.4,
                  size = 1) +
  p9.theme_classic() +
  p9.theme(plot_background = p9.element_rect(fill = "white", alpha = 0),
           panel_background = p9.element_rect(fill = "white", alpha = 0),
           panel_border = p9.element_rect(fill = "white", alpha = 0),
           legend_position = (0.9, 0.6)) +
  p9.scale_x_reverse() +
  p9.scale_y_continuous(expand = (0, 0, 0, 50)) +
  p9.scale_fill_manual(values = [iso_color[5], iso_color[0]],
                       labels = ["C$_2$D$_5$COO$^-$", "C$_2$H$_5$COO$^-$"]) +
  p9.guides(color = False) +
  p9.annotate('text', x = 3075, y = 60, label = "C–H") +
  p9.annotate('text', x = 2250, y = 60, label = "C–D") +
  p9.annotate('text', x = 1600, y = 650, label = "C=O") +
  p9.annotate('text', x = 1400, y = 275, label = "C=O") +
  p9.xlab("Wavenumber (cm$^{-1}$)") +
  p9.ylab("Intensity (arbitrary units)")
)

p9.ggsave(plot, filename = "vib_all-h-or-d_withgasphase.pdf", 
          width = 12, height = 6, units = "in", dpi = 300)