################################################################################
####                              vib_average.R                             ####
################################################################################

library(tidyverse)
library(docstring)

#' Data workup of computational vibrational spectra of propionate in
#' (classical) OR51E2 protein.
#' In this file, we take the simulated vibrational frequencies/IR intensities
#'  for all 32 possible H/D isotope isomers of C2H5COO-,
#'  at 20 ns intervals in the MD trajectory (see below).
#' The IR spectra are averaged over time (assuming normal distribution since
#'  they are so far separated: i.e., we take the stdev for Gaussian broadening)
#' Experimental structure and MD trajectory:
#'    https://doi.org/10.1038/s41586-023-05798-y,
#'    deposited at https://www.gpcrmd.org/view/1245/
#' Computational setup:
#'    QM: B3LYP-D4; aug-cc-pVTZ basis set. QM region is ligand only;
#'        only ligand's geometry and vibrational spectrum are obtained.
#'        Spectra for C2H5COO- and C2D5COO-.
#'    MM: CHARMM36m (CGenFF used to generate ligand topology).

# Change as needed!
sourcedir <- file.path("/Users/celerity/Library/CloudStorage/Box-Box",
                       "Home Folder jzw5/Research/quantum-olfaction",
                       "paper/data/data-plot")
setwd(file.path(sourcedir))
#setwd("./")

# Importing all CSVs: from https://stackoverflow.com/questions/11433432
dat <- list.files(pattern = "\\.csv$") %>% 
  map_df(~read_csv(., col_names = TRUE, id = "name"))

# Clean up the data
dat_test <- dat %>% 
  rename(freq = `freq_cm-1`, intensity = `IRint_km/mol`) %>%
  separate_wider_delim(name, delim = "_", 
                       names = c("time", "isotope")) %>%
  mutate(time = str_remove_all(time, "ns"),
         isotope = str_remove_all(isotope, ".csv")) %>%
  group_by(time, isotope) %>%
  mutate(peak = row_number(time)) %>%
  select(peak, isotope, time, freq, intensity)

# Remove translational/rotational modes (6 of lowest-frequency),
# and average each peak over the trajectory snapshots.
# Also collect 
avg_ir <- dat_test %>% filter(peak > 6) %>%
  group_by(isotope, peak) %>%
  summarise(freqmean = mean(freq), freqsd = sd(freq),
            intmean = mean(intensity), intsd = sd(intensity))

# Write to csv
write_csv(avg_ir, "avg_ir.csv")

#### Now, do the same for the gas-phase data. (No need for averaging!)
propdir <- file.path("/Users/celerity/Library/CloudStorage/Box-Box",
                       "Home Folder jzw5/Research/quantum-olfaction",
                       "paper/data/data-plot/gas-phase-propionate")
setwd(file.path(propdir))

dat <- list.files(pattern = "\\.csv$") %>% 
  map_df(~read_csv(., col_names = TRUE, id = "name"))

dat_test <- dat %>% 
  rename(freq = `freq_cm-1`, intensity = `IRint_km/mol`) %>%
  separate_wider_delim(name, delim = "_", 
                       names = c("species", "vib", "isotope")) %>%
  mutate(isotope = str_remove_all(isotope, ".csv")) %>%
  group_by(species, isotope) %>%
  mutate(peak = row_number(species)) %>%
  select(species, peak, isotope, freq, intensity)

avg_ir_gasphase <- dat_test %>% filter(peak > 6)

write_csv(avg_ir_gasphase, "avg_ir_gasphase.csv")
